@@ -30,30 +30,84 @@
 
 #include "rdsysqueue.h"
 
+#include "rdkafka_proto.h"
+
+
+/**
+ * Message.Attributes
+ */
+#define RD_KAFKA_MSG_ATTR_GZIP             (1 << 0)
+#define RD_KAFKA_MSG_ATTR_SNAPPY           (1 << 1)
+#define RD_KAFKA_MSG_ATTR_LZ4              (1 << 2)
+#define RD_KAFKA_MSG_ATTR_COMPRESSION_MASK 0x7
+#define RD_KAFKA_MSG_ATTR_CREATE_TIME      (0 << 3)
+#define RD_KAFKA_MSG_ATTR_LOG_APPEND_TIME  (1 << 3)
+
+
+typedef struct rd_kafka_msg_s {
+	TAILQ_ENTRY(rd_kafka_msg_s)  rkm_link;
+	int        rkm_flags;
+	size_t     rkm_len;
+	void      *rkm_payload;
+	void      *rkm_opaque;
+	int32_t    rkm_partition;  /* partition specified */
+	rd_kafkap_bytes_t *rkm_key;
+        int64_t    rkm_offset;
+	int64_t    rkm_timestamp;  /* Message format V1.
+				    * Meaning of timestamp depends on
+				    * message Attribute LogAppendtime (broker)
+				    * or CreateTime (producer).
+				    * Unit is milliseconds since epoch (UTC).*/
+	rd_ts_t    rkm_ts_timeout;
+} rd_kafka_msg_t;
+
+TAILQ_HEAD(rd_kafka_msg_head_s, rd_kafka_msg_s);
+
+typedef struct rd_kafka_msgq_s {
+	TAILQ_HEAD(, rd_kafka_msg_s) rkmq_msgs;
+	rd_atomic32_t rkmq_msg_cnt;
+	rd_atomic64_t rkmq_msg_bytes;
+} rd_kafka_msgq_t;
+
+#define RD_KAFKA_MSGQ_INITIALIZER(rkmq) \
+	{ .rkmq_msgs = TAILQ_HEAD_INITIALIZER((rkmq).rkmq_msgs) }
+
+#define RD_KAFKA_MSGQ_FOREACH(elm,head) \
+	TAILQ_FOREACH(elm, &(head)->rkmq_msgs, rkm_link)
+
+/**
+ * Returns the number of messages in the specified queue.
+ */
+static RD_INLINE RD_UNUSED int rd_kafka_msgq_len (rd_kafka_msgq_t *rkmq) {
+	return (int)rd_atomic32_get(&rkmq->rkmq_msg_cnt);
+}
+
+
 void rd_kafka_msg_destroy (rd_kafka_t *rk, rd_kafka_msg_t *rkm);
 
-int rd_kafka_msg_new (rd_kafka_topic_t *rkt, int32_t force_partition,
+int rd_kafka_msg_new (rd_kafka_itopic_t *rkt, int32_t force_partition,
 		      int msgflags,
 		      char *payload, size_t len,
 		      const void *keydata, size_t keylen,
 		      void *msg_opaque);
 
 
-static inline RD_UNUSED void rd_kafka_msgq_init (rd_kafka_msgq_t *rkmq) {
+static RD_INLINE RD_UNUSED void rd_kafka_msgq_init (rd_kafka_msgq_t *rkmq) {
 	TAILQ_INIT(&rkmq->rkmq_msgs);
-	rkmq->rkmq_msg_cnt   = 0;
-	rkmq->rkmq_msg_bytes = 0;
+	rd_atomic32_set(&rkmq->rkmq_msg_cnt, 0);
+	rd_atomic64_set(&rkmq->rkmq_msg_bytes, 0);
 }
 
 /**
  * Concat all elements of 'src' onto tail of 'dst'.
  * 'src' will be cleared.
+ * Proper locks for 'src' and 'dst' must be held.
  */
-static inline RD_UNUSED void rd_kafka_msgq_concat (rd_kafka_msgq_t *dst,
+static RD_INLINE RD_UNUSED void rd_kafka_msgq_concat (rd_kafka_msgq_t *dst,
 						   rd_kafka_msgq_t *src) {
 	TAILQ_CONCAT(&dst->rkmq_msgs, &src->rkmq_msgs, rkm_link);
-	(void)rd_atomic_add(&dst->rkmq_msg_cnt, src->rkmq_msg_cnt);
-	(void)rd_atomic_add(&dst->rkmq_msg_bytes, src->rkmq_msg_bytes);
+	(void)rd_atomic32_add(&dst->rkmq_msg_cnt, rd_atomic32_get(&src->rkmq_msg_cnt));
+	(void)rd_atomic64_add(&dst->rkmq_msg_bytes, rd_atomic64_get(&src->rkmq_msg_bytes));
 	rd_kafka_msgq_init(src);
 }
 
@@ -61,20 +115,20 @@ static inline RD_UNUSED void rd_kafka_msgq_concat (rd_kafka_msgq_t *dst,
  * Move queue 'src' to 'dst' (overwrites dst)
  * Source will be cleared.
  */
-static inline RD_UNUSED void rd_kafka_msgq_move (rd_kafka_msgq_t *dst,
+static RD_INLINE RD_UNUSED void rd_kafka_msgq_move (rd_kafka_msgq_t *dst,
 						 rd_kafka_msgq_t *src) {
 	TAILQ_MOVE(&dst->rkmq_msgs, &src->rkmq_msgs, rkm_link);
-	dst->rkmq_msg_cnt   = src->rkmq_msg_cnt;
-	dst->rkmq_msg_bytes = src->rkmq_msg_bytes;
+	rd_atomic32_set(&dst->rkmq_msg_cnt, rd_atomic32_get(&src->rkmq_msg_cnt));
+	rd_atomic64_set(&dst->rkmq_msg_bytes, rd_atomic64_get(&src->rkmq_msg_bytes));
 	rd_kafka_msgq_init(src);
 }
 
 
 /**
- * Free all msgs in msgq and reinitialize the msgq.
+ * rd_free all msgs in msgq and reinitialize the msgq.
  */
-static inline RD_UNUSED void rd_kafka_msgq_purge (rd_kafka_t *rk,
-						  rd_kafka_msgq_t *rkmq) {
+static RD_INLINE RD_UNUSED void rd_kafka_msgq_purge (rd_kafka_t *rk,
+                                                    rd_kafka_msgq_t *rkmq) {
 	rd_kafka_msg_t *rkm, *next;
 
 	next = TAILQ_FIRST(&rkmq->rkmq_msgs);
@@ -92,15 +146,15 @@ static inline RD_UNUSED void rd_kafka_msgq_purge (rd_kafka_t *rk,
 /**
  * Remove message from message queue
  */
-static inline RD_UNUSED 
+static RD_INLINE RD_UNUSED 
 rd_kafka_msg_t *rd_kafka_msgq_deq (rd_kafka_msgq_t *rkmq,
 				   rd_kafka_msg_t *rkm,
 				   int do_count) {
 	if (likely(do_count)) {
-		rd_kafka_assert(NULL, rkmq->rkmq_msg_cnt > 0);
-		rd_kafka_assert(NULL, rkmq->rkmq_msg_bytes >= rkm->rkm_len);
-		(void)rd_atomic_sub(&rkmq->rkmq_msg_cnt, 1);
-		(void)rd_atomic_sub(&rkmq->rkmq_msg_bytes, rkm->rkm_len);
+		rd_kafka_assert(NULL, rd_atomic32_get(&rkmq->rkmq_msg_cnt) > 0);
+		rd_kafka_assert(NULL, rd_atomic64_get(&rkmq->rkmq_msg_bytes) >= (int64_t)rkm->rkm_len);
+		(void)rd_atomic32_sub(&rkmq->rkmq_msg_cnt, 1);
+		(void)rd_atomic64_sub(&rkmq->rkmq_msg_bytes, rkm->rkm_len);
 	}
 
 	TAILQ_REMOVE(&rkmq->rkmq_msgs, rkm, rkm_link);
@@ -108,7 +162,7 @@ rd_kafka_msg_t *rd_kafka_msgq_deq (rd_kafka_msgq_t *rkmq,
 	return rkm;
 }
 
-static inline RD_UNUSED
+static RD_INLINE RD_UNUSED
 rd_kafka_msg_t *rd_kafka_msgq_pop (rd_kafka_msgq_t *rkmq) {
 	rd_kafka_msg_t *rkm;
 
@@ -121,21 +175,21 @@ rd_kafka_msg_t *rd_kafka_msgq_pop (rd_kafka_msgq_t *rkmq) {
 /**
  * Insert message at head of message queue.
  */
-static inline RD_UNUSED void rd_kafka_msgq_insert (rd_kafka_msgq_t *rkmq,
+static RD_INLINE RD_UNUSED void rd_kafka_msgq_insert (rd_kafka_msgq_t *rkmq,
 						   rd_kafka_msg_t *rkm) {
 	TAILQ_INSERT_HEAD(&rkmq->rkmq_msgs, rkm, rkm_link);
-	(void)rd_atomic_add(&rkmq->rkmq_msg_cnt, 1);
-	(void)rd_atomic_add(&rkmq->rkmq_msg_bytes, rkm->rkm_len);
+	(void)rd_atomic32_add(&rkmq->rkmq_msg_cnt, 1);
+	(void)rd_atomic64_add(&rkmq->rkmq_msg_bytes, rkm->rkm_len);
 }
 
 /**
  * Append message to tail of message queue.
  */
-static inline RD_UNUSED void rd_kafka_msgq_enq (rd_kafka_msgq_t *rkmq,
+static RD_INLINE RD_UNUSED void rd_kafka_msgq_enq (rd_kafka_msgq_t *rkmq,
 						rd_kafka_msg_t *rkm) {
 	TAILQ_INSERT_TAIL(&rkmq->rkmq_msgs, rkm, rkm_link);
-	(void)rd_atomic_add(&rkmq->rkmq_msg_cnt, 1);
-	(void)rd_atomic_add(&rkmq->rkmq_msg_bytes, rkm->rkm_len);
+	(void)rd_atomic32_add(&rkmq->rkmq_msg_cnt, 1);
+	(void)rd_atomic64_add(&rkmq->rkmq_msg_bytes, rkm->rkm_len);
 }
 
 
@@ -150,5 +204,5 @@ int rd_kafka_msgq_age_scan (rd_kafka_msgq_t *rkmq,
 			    rd_ts_t now);
 
 
-int rd_kafka_msg_partitioner (rd_kafka_topic_t *rkt, rd_kafka_msg_t *rkm,
+int rd_kafka_msg_partitioner (rd_kafka_itopic_t *rkt, rd_kafka_msg_t *rkm,
 			      int do_lock);
