@@ -30,9 +30,6 @@
  * Tests various config related things
  */
 
-#define _GNU_SOURCE
-#include <sys/time.h>
-#include <time.h>
 
 #include "test.h"
 
@@ -68,7 +65,7 @@ static void conf_verify (int line,
 
 
 	for (i = 0 ; confs[i] ; i += 2) {
-		for (j = 0 ; j < cnt ; j += 2) {
+		for (j = 0 ; j < (int)cnt ; j += 2) {
 			if (!strcmp(confs[i], arr[j])) {
 				if (strcmp(confs[i+1], arr[j+1]))
 					TEST_FAIL("%i: Property %s mismatch: "
@@ -77,7 +74,7 @@ static void conf_verify (int line,
 						  confs[i],
 						  confs[i+1], arr[j+1]);
 			}
-			if (j == cnt)
+			if (j == (int)cnt)
 				TEST_FAIL("%i: "
 					  "Property %s not found in config\n",
 					  line,
@@ -96,7 +93,7 @@ static void conf_cmp (const char *desc,
 		TEST_FAIL("%s config compare: count %zd != %zd mismatch",
 			  desc, acnt, bcnt);
 
-	for (i = 0 ; i < acnt ; i += 2) {
+	for (i = 0 ; i < (int)acnt ; i += 2) {
 		if (strcmp(a[i], b[i]))
 			TEST_FAIL("%s conf mismatch: %s != %s",
 				  desc, a[i], b[i]);
@@ -107,7 +104,7 @@ static void conf_cmp (const char *desc,
 }
 
 
-int main (int argc, char **argv) {
+int main_0004_conf (int argc, char **argv) {
 	rd_kafka_t *rk;
 	rd_kafka_topic_t *rkt;
 	rd_kafka_conf_t *ignore_conf, *conf, *conf2;
@@ -121,7 +118,9 @@ int main (int argc, char **argv) {
 		"message.max.bytes", "12345", /* int property */
 		"client.id", "my id", /* string property */
 		"debug", "topic,metadata", /* S2F property */
+#if WITH_ZLIB
 		"compression.codec", "gzip", /* S2I property */
+#endif
 		NULL
 	};
 	static const char *tconfs[] = {
@@ -136,7 +135,7 @@ int main (int argc, char **argv) {
 	rd_kafka_conf_destroy(ignore_conf);
 	rd_kafka_topic_conf_destroy(ignore_topic_conf);
 
-        topic = test_mk_topic_name("generic", 0);
+        topic = test_mk_topic_name("0004", 0);
 
 	/* Set up a global config object */
 	conf = rd_kafka_conf_new();
@@ -203,7 +202,7 @@ int main (int argc, char **argv) {
 	rkt = rd_kafka_topic_new(rk, topic, tconf);
 	if (!rkt)
 		TEST_FAIL("Failed to create topic: %s\n",
-			  strerror(errno));
+			  rd_strerror(errno));
 
 	rd_kafka_topic_destroy(rkt);
 	rd_kafka_destroy(rk);
@@ -217,17 +216,131 @@ int main (int argc, char **argv) {
 	rkt = rd_kafka_topic_new(rk, topic, tconf2);
 	if (!rkt)
 		TEST_FAIL("Failed to create topic: %s\n",
-			  strerror(errno));
+			  rd_strerror(errno));
 
 	rd_kafka_topic_destroy(rkt);
 	rd_kafka_destroy(rk);
 
 
-	/* Wait for everything to be cleaned up since broker destroys are
-	 * handled in its own thread. */
-	test_wait_exit(2);
+	/* Incremental S2F property.
+	 * NOTE: The order of fields returned in get() is hardcoded here. */
+	{
+		static const char *s2fs[] = {
+			"generic,broker,queue,cgrp",
+			"generic,broker,queue,cgrp",
+
+			"-broker,+queue,topic",
+			"generic,topic,queue,cgrp",
+
+			"-all,security,-fetch,+metadata",
+			"metadata,security",
+
+			NULL
+		};
+
+		TEST_SAY("Incremental S2F tests\n");
+		conf = rd_kafka_conf_new();
+
+		for (i = 0 ; s2fs[i] ; i += 2) {
+			const char *val;
+
+			TEST_SAY("  Set: %s\n", s2fs[i]);
+			test_conf_set(conf, "debug", s2fs[i]);
+			val = test_conf_get(conf, "debug");
+			TEST_SAY("  Now: %s\n", val);
+
+			if (strcmp(val, s2fs[i+1]))
+				TEST_FAIL_LATER("\n"
+						"Expected: %s\n"
+						"     Got: %s",
+						s2fs[i+1], val);
+		}
+		rd_kafka_conf_destroy(conf);
+	}
+
+	/* Canonical int values, aliases, s2i-verified strings */
+	{
+		static const struct {
+			const char *prop;
+			const char *val;
+			const char *exp;
+			int is_global;
+		} props[] = {
+			{ "request.required.acks", "0", "0" },
+			{ "request.required.acks", "-1", "-1" },
+			{ "request.required.acks", "1", "1" },
+			{ "acks", "3", "3" }, /* alias test */
+			{ "request.required.acks", "393", "393" },
+			{ "request.required.acks", "bad", NULL },
+			{ "request.required.acks", "all", "-1" },
+			{ "acks", "0", "0" }, /* alias test */
+#if WITH_SASL
+			{ "sasl.mechanisms", "GSSAPI", "GSSAPI", 1 },
+			{ "sasl.mechanisms", "PLAIN", "PLAIN", 1  },
+			{ "sasl.mechanisms", "GSSAPI,PLAIN", NULL, 1  },
+			{ "sasl.mechanisms", "", NULL, 1  },
+			{ "sasl.mechanisms", "foobar", NULL, 1  },
+			{ "sasl.mechanisms", "plain", "PLAIN", 1  },
+#endif
+			{ NULL }
+		};
+
+		TEST_SAY("Canonical tests\n");
+		tconf = rd_kafka_topic_conf_new();
+		conf = rd_kafka_conf_new();
+
+		for (i = 0 ; props[i].prop ; i++) {
+			char dest[64];
+			size_t destsz;
+			rd_kafka_conf_res_t res;
+
+			TEST_SAY("  Set: %s=%s expect %s\n",
+				 props[i].prop, props[i].val, props[i].exp);
+
+
+			/* Set value */
+			if (props[i].is_global)
+				res = rd_kafka_conf_set(conf,
+						      props[i].prop,
+						      props[i].val,
+						      errstr, sizeof(errstr));
+			else
+				res = rd_kafka_topic_conf_set(tconf,
+							      props[i].prop,
+							      props[i].val,
+							      errstr,
+							      sizeof(errstr));
+			if ((res == RD_KAFKA_CONF_OK ? 1:0) !=
+			    (props[i].exp ? 1:0))
+				TEST_FAIL("Expected %s, got %s",
+					  props[i].exp ? "success" : "failure",
+					  (res == RD_KAFKA_CONF_OK ? "OK" :
+					   (res == RD_KAFKA_CONF_INVALID ? "INVALID" :
+					    "UNKNOWN")));
+
+			if (!props[i].exp)
+				continue;
+
+			/* Get value and compare to expected result */
+			destsz = sizeof(dest);
+			if (props[i].is_global)
+				res = rd_kafka_conf_get(conf,
+							props[i].prop,
+							dest, &destsz);
+			else
+				res = rd_kafka_topic_conf_get(tconf,
+							      props[i].prop,
+							      dest, &destsz);
+			TEST_ASSERT(res == RD_KAFKA_CONF_OK,
+				    ".._conf_get() returned %d", res);
+
+			TEST_ASSERT(!strcmp(props[i].exp, dest),
+				    "Expected \"%s\", got \"%s\"",
+				    props[i].exp, dest);
+		}
+		rd_kafka_topic_conf_destroy(tconf);
+		rd_kafka_conf_destroy(conf);
+	}
 
-	/* If we havent failed at this point then
-	 * there were no threads leaked */
 	return 0;
 }
